@@ -5,7 +5,7 @@
  * @brief libnetconf2 - input/output functions
  *
  * @copyright
- * Copyright (c) 2015 - 2024 CESNET, z.s.p.o.
+ * Copyright (c) 2015 - 2025 CESNET, z.s.p.o.
  *
  * This source code is licensed under BSD 3-Clause License (the "License").
  * You may not use this file except in compliance with the License.
@@ -18,6 +18,7 @@
 
 #include <assert.h>
 #include <errno.h>
+#include <grp.h>
 #include <inttypes.h>
 #include <limits.h>
 #include <pwd.h>
@@ -184,14 +185,13 @@ nc_read(struct nc_session *session, char *buf, uint32_t count, uint32_t inact_ti
  * @return Number of characters read;
  * @return -1 on error.
  */
-static ssize_t
+static int
 nc_read_until(struct nc_session *session, const char *endtag, uint32_t inact_timeout, struct timespec *ts_act_timeout,
         char **buf, uint32_t *buf_size)
 {
     char *local_buf = NULL;
+    int r, len, i, matched = 0, count = 0;
     uint32_t local_size = 0;
-    size_t r, len, i, matched = 0;
-    ssize_t count = 0;
 
     assert(session);
     assert(endtag);
@@ -204,7 +204,7 @@ nc_read_until(struct nc_session *session, const char *endtag, uint32_t inact_tim
     len = strlen(endtag);
     do {
         /* resize buffer if needed */
-        if ((count + (len - matched)) > *buf_size) {
+        if ((count + (len - matched)) > (signed)*buf_size) {
             *buf_size += NC_READ_BUF_SIZE_STEP;
             *buf = nc_realloc(*buf, (*buf_size + 1) * sizeof **buf);
             NC_CHECK_ERRMEM_GOTO(!*buf, count = -1, cleanup);
@@ -241,15 +241,14 @@ cleanup:
 }
 
 int
-nc_read_msg_io(struct nc_session *session, int io_timeout, struct ly_in **msg, int passing_io_lock)
+nc_read_msg_io(struct nc_session *session, int io_timeout, int passing_io_lock, char **buf, uint32_t *buf_len)
 {
-    int ret = 1, r, io_locked = passing_io_lock;
-    char *data = NULL, *frame_size_buf = NULL;
-    uint32_t chunk_len, frame_buf_len, data_len = 0, inact_timeout;
+    int ret = -1, r, io_locked = 0, chunk_len, buf_used = 0;
+    char *frame_size_buf = NULL;
+    uint32_t inact_timeout, frame_buf_len;
     struct timespec ts_act_timeout;
 
-    assert(session && msg);
-    *msg = NULL;
+    assert(session && buf && buf_len);
 
     /* use timeout in milliseconds instead seconds */
     inact_timeout = NC_READ_INACT_TIMEOUT * 1000;
@@ -262,7 +261,7 @@ nc_read_msg_io(struct nc_session *session, int io_timeout, struct ly_in **msg, i
 
     nc_timeouttime_get(&ts_act_timeout, NC_READ_ACT_TIMEOUT * 1000);
 
-    if (!io_locked) {
+    if (!passing_io_lock) {
         /* SESSION IO LOCK */
         ret = nc_session_io_lock(session, io_timeout, __func__);
         if (ret < 1) {
@@ -273,17 +272,18 @@ nc_read_msg_io(struct nc_session *session, int io_timeout, struct ly_in **msg, i
 
     /* read the message */
     switch (session->version) {
-    case NC_VERSION_10:
-        r = nc_read_until(session, NC_VERSION_10_ENDTAG, inact_timeout, &ts_act_timeout, &data, &data_len);
+    case NC_PROT_VERSION_10:
+        r = nc_read_until(session, NC_VERSION_10_ENDTAG, inact_timeout, &ts_act_timeout, buf, buf_len);
         if (r == -1) {
             ret = r;
             goto cleanup;
         }
 
         /* cut off the end tag */
-        data[r - NC_VERSION_10_ENDTAG_LEN] = '\0';
+        ret = r - NC_VERSION_10_ENDTAG_LEN;
+        (*buf)[ret] = '\0';
         break;
-    case NC_VERSION_11:
+    case NC_PROT_VERSION_11:
         /* prepare the buffer large enough for reading the framing chunk size */
         frame_buf_len = 11;
         frame_size_buf = malloc(frame_buf_len + 1);
@@ -303,7 +303,7 @@ nc_read_msg_io(struct nc_session *session, int io_timeout, struct ly_in **msg, i
 
             if (!strcmp(frame_size_buf, "#\n")) {
                 /* end of chunked framing message */
-                if (!data) {
+                if (!buf_used) {
                     ERR(session, "Invalid frame chunk delimiters.");
                     ret = -2;
                     goto cleanup;
@@ -320,37 +320,29 @@ nc_read_msg_io(struct nc_session *session, int io_timeout, struct ly_in **msg, i
             }
 
             /* now we have size of next chunk, prepare a buffer large enough */
-            data = nc_realloc(data, data_len + chunk_len + 1);
-            NC_CHECK_ERRMEM_GOTO(!data, ret = -1, cleanup);
+            if ((signed)*buf_len < buf_used + chunk_len + 1) {
+                *buf_len = buf_used + chunk_len + 1;
+                *buf = nc_realloc(*buf, *buf_len);
+                NC_CHECK_ERRMEM_GOTO(!*buf, ret = -1, cleanup);
+            }
 
             /* read the next chunk */
-            r = nc_read(session, data + data_len, chunk_len, inact_timeout, &ts_act_timeout);
+            r = nc_read(session, *buf + buf_used, chunk_len, inact_timeout, &ts_act_timeout);
             if (r == -1) {
                 ret = r;
                 goto cleanup;
             }
 
             /* update data length and terminate the data */
-            data_len += chunk_len;
-            data[data_len] = '\0';
+            buf_used += chunk_len;
+            (*buf)[buf_used] = '\0';
         }
 
+        ret = buf_used;
         break;
     }
 
-    /* SESSION IO UNLOCK */
-    assert(io_locked);
-    nc_session_io_unlock(session, __func__);
-    io_locked = 0;
-
-    DBG(session, "Received message:\n%s\n", data);
-
-    /* build an input structure, eats data */
-    if (ly_in_new_memory(data, msg)) {
-        ret = -1;
-        goto cleanup;
-    }
-    data = NULL;
+    DBG(session, "Received message:\n%s\n", *buf);
 
 cleanup:
     if (io_locked) {
@@ -358,7 +350,6 @@ cleanup:
         nc_session_io_unlock(session, __func__);
     }
     free(frame_size_buf);
-    free(data);
     return ret;
 }
 
@@ -440,7 +431,7 @@ nc_read_poll(struct nc_session *session, int io_timeout)
         session->status = NC_STATUS_INVALID;
         session->term_reason = NC_SESSION_TERM_OTHER;
         return -1;
-    } else { /* status > 0 */
+    } else {
         /* in case of standard (non-libssh) poll, there still can be an error */
         if (fds.revents & POLLERR) {
             ERR(session, "Communication channel error.");
@@ -465,6 +456,8 @@ int
 nc_read_msg_poll_io(struct nc_session *session, int io_timeout, struct ly_in **msg)
 {
     int ret;
+    uint32_t buf_len = 0;
+    char *buf = NULL;
 
     assert(msg);
     *msg = NULL;
@@ -489,8 +482,26 @@ nc_read_msg_poll_io(struct nc_session *session, int io_timeout, struct ly_in **m
         return ret;
     }
 
-    /* SESSION IO LOCK passed down */
-    return nc_read_msg_io(session, io_timeout, msg, 1);
+    /* read msg */
+    ret = nc_read_msg_io(session, io_timeout, 1, &buf, &buf_len);
+
+    /* SESSION IO UNLOCK */
+    nc_session_io_unlock(session, __func__);
+
+    if (ret > 0) {
+        ret = 1;
+    } else {
+        free(buf);
+        return ret;
+    }
+
+    /* create input */
+    if (ly_in_new_memory(buf, msg)) {
+        free(buf);
+        return -1;
+    }
+
+    return ret;
 }
 
 /* does not really log, only fatal errors */
@@ -535,12 +546,6 @@ nc_session_is_connected(const struct nc_session *session)
     return 1;
 }
 
-struct nc_wclb_arg {
-    struct nc_session *session;
-    char buf[NC_WRITE_CHUNK_SIZE_MAX];
-    uint32_t len;
-};
-
 /**
  * @brief Write to a NETCONF session.
  *
@@ -568,7 +573,7 @@ nc_write(struct nc_session *session, const void *buf, uint32_t count)
         return -1;
     }
 
-    DBG(session, "Sending message:\n%.*s\n", (int)count, buf);
+    DBG(session, "Sending message:\n%.*s\n", (int)count, (char *)buf);
 
     do {
         interrupted = 0;
@@ -576,8 +581,8 @@ nc_write(struct nc_session *session, const void *buf, uint32_t count)
         case NC_TI_FD:
         case NC_TI_UNIX:
             fd = session->ti_type == NC_TI_FD ? session->ti.fd.out : session->ti.unixsock.sock;
-            c = write(fd, (char *)(buf + written), count - written);
-            if ((c < 0) && (errno == EAGAIN)) {
+            c = write(fd, (char *)buf + written, count - written);
+            if ((c < 0) && ((errno == EAGAIN) || (errno == EWOULDBLOCK))) {
                 c = 0;
             } else if ((c < 0) && (errno == EINTR)) {
                 c = 0;
@@ -600,14 +605,14 @@ nc_write(struct nc_session *session, const void *buf, uint32_t count)
                 session->term_reason = NC_SESSION_TERM_DROPPED;
                 return -1;
             }
-            c = ssh_channel_write(session->ti.libssh.channel, (char *)(buf + written), count - written);
+            c = ssh_channel_write(session->ti.libssh.channel, (char *)buf + written, count - written);
             if ((c == SSH_ERROR) || (c == -1)) {
                 ERR(session, "SSH channel write failed.");
                 return -1;
             }
             break;
         case NC_TI_TLS:
-            c = nc_tls_write_wrap(session, (const unsigned char *)(buf + written), count - written);
+            c = nc_tls_write_wrap(session, (const unsigned char *)buf + written, count - written);
             if (c < 0) {
                 /* possible client dc, or some socket/TLS communication error */
                 return -1;
@@ -645,18 +650,18 @@ nc_write_starttag_and_msg(struct nc_session *session, const void *buf, uint32_t
     int ret = 0, r;
     char chunksize[24];
 
-    if (session->version == NC_VERSION_11) {
+    if (session->version == NC_PROT_VERSION_11) {
         r = sprintf(chunksize, "\n#%" PRIu32 "\n", count);
 
         r = nc_write(session, chunksize, r);
-        if (r == -1) {
+        if (r < 0) {
             return -1;
         }
         ret += r;
     }
 
     r = nc_write(session, buf, count);
-    if (r == -1) {
+    if (r < 0) {
         return -1;
     }
     ret += r;
@@ -676,7 +681,7 @@ nc_write_endtag(struct nc_session *session)
 {
     int ret;
 
-    if (session->version == NC_VERSION_11) {
+    if (session->version == NC_PROT_VERSION_11) {
         ret = nc_write(session, "\n##\n", 4);
     } else {
         ret = nc_write(session, "]]>]]>", 6);
@@ -706,32 +711,21 @@ nc_write_clb_flush(struct nc_wclb_arg *warg)
     return ret;
 }
 
-/**
- * @brief Write callback buffering the data in a write structure.
- *
- * @param[in] arg Write structure used for buffering.
- * @param[in] buf Buffer to write.
- * @param[in] count Count of bytes to write from @p buf.
- * @param[in] xmlcontent Whether the data are actually printed as part of an XML in which case they need to be encoded.
- * @return Number of written bytes.
- * @return -1 on error.
- */
-static ssize_t
-nc_write_clb(void *arg, const void *buf, uint32_t count, int xmlcontent)
+int
+nc_write_clb(struct nc_wclb_arg *arg, const void *buf, uint32_t count, int xmlcontent)
 {
-    ssize_t ret = 0, c;
+    int ret = 0, c;
     uint32_t l;
-    struct nc_wclb_arg *warg = arg;
 
     if (!buf) {
-        c = nc_write_clb_flush(warg);
+        c = nc_write_clb_flush(arg);
         if (c == -1) {
             return -1;
         }
         ret += c;
 
         /* endtag */
-        c = nc_write_endtag(warg->session);
+        c = nc_write_endtag(arg->session);
         if (c == -1) {
             return -1;
         }
@@ -740,9 +734,9 @@ nc_write_clb(void *arg, const void *buf, uint32_t count, int xmlcontent)
         return ret;
     }
 
-    if (warg->len && (warg->len + count > NC_WRITE_CHUNK_SIZE_MAX)) {
+    if (arg->len && (arg->len + count > NC_WRITE_CHUNK_SIZE_MAX)) {
         /* dump current buffer */
-        c = nc_write_clb_flush(warg);
+        c = nc_write_clb_flush(arg);
         if (c == -1) {
             return -1;
         }
@@ -751,7 +745,7 @@ nc_write_clb(void *arg, const void *buf, uint32_t count, int xmlcontent)
 
     if (!xmlcontent && (count > NC_WRITE_CHUNK_SIZE_MAX)) {
         /* write directly */
-        c = nc_write_starttag_and_msg(warg->session, buf, count);
+        c = nc_write_starttag_and_msg(arg->session, buf, count);
         if (c == -1) {
             return -1;
         }
@@ -760,9 +754,9 @@ nc_write_clb(void *arg, const void *buf, uint32_t count, int xmlcontent)
         /* keep in buffer and write later */
         if (xmlcontent) {
             for (l = 0; l < count; l++) {
-                if (warg->len + 5 >= NC_WRITE_CHUNK_SIZE_MAX) {
+                if (arg->len + 5 >= NC_WRITE_CHUNK_SIZE_MAX) {
                     /* buffer is full */
-                    c = nc_write_clb_flush(warg);
+                    c = nc_write_clb_flush(arg);
                     if (c == -1) {
                         return -1;
                     }
@@ -771,29 +765,29 @@ nc_write_clb(void *arg, const void *buf, uint32_t count, int xmlcontent)
                 switch (((char *)buf)[l]) {
                 case '&':
                     ret += 5;
-                    memcpy(&warg->buf[warg->len], "&amp;", 5);
-                    warg->len += 5;
+                    memcpy(&arg->buf[arg->len], "&amp;", 5);
+                    arg->len += 5;
                     break;
                 case '<':
                     ret += 4;
-                    memcpy(&warg->buf[warg->len], "&lt;", 4);
-                    warg->len += 4;
+                    memcpy(&arg->buf[arg->len], "&lt;", 4);
+                    arg->len += 4;
                     break;
                 case '>':
                     /* not needed, just for readability */
                     ret += 4;
-                    memcpy(&warg->buf[warg->len], "&gt;", 4);
-                    warg->len += 4;
+                    memcpy(&arg->buf[arg->len], "&gt;", 4);
+                    arg->len += 4;
                     break;
                 default:
                     ret++;
-                    memcpy(&warg->buf[warg->len], &((char *)buf)[l], 1);
-                    warg->len++;
+                    memcpy(&arg->buf[arg->len], &((char *)buf)[l], 1);
+                    arg->len++;
                 }
             }
         } else {
-            memcpy(&warg->buf[warg->len], buf, count);
-            warg->len += count; /* is <= NC_WRITE_CHUNK_SIZE_MAX */
+            memcpy(&arg->buf[arg->len], buf, count);
+            arg->len += count; /* is <= NC_WRITE_CHUNK_SIZE_MAX */
             ret += count;
         }
     }
@@ -874,7 +868,7 @@ nc_write_msg_io(struct nc_session *session, int io_timeout, int type, ...)
         }
 
         /* rpc data */
-        if (lyd_print_clb(nc_write_xmlclb, (void *)&arg, op, LYD_XML, LYD_PRINT_SHRINK | LYD_PRINT_KEEPEMPTYCONT)) {
+        if (lyd_print_clb(nc_write_xmlclb, (void *)&arg, op, LYD_XML, LYD_PRINT_SHRINK | LYD_PRINT_EMPTY_CONT)) {
             ret = NC_MSG_ERROR;
             goto cleanup;
         }
@@ -1011,7 +1005,7 @@ nc_write_msg_io(struct nc_session *session, int io_timeout, int type, ...)
         break;
 
     case NC_MSG_HELLO:
-        if (session->version != NC_VERSION_10) {
+        if (session->version != NC_PROT_VERSION_10) {
             ret = NC_MSG_ERROR;
             goto cleanup;
         }
@@ -1100,12 +1094,54 @@ nc_getpw(uid_t uid, const char *username, struct passwd *pwd_buf, char **buf, si
         }
     } while (ret && (ret == ERANGE));
 
-    if (ret) {
+    if (ret || !pwd) {
         if (username) {
-            ERR(NULL, "Retrieving username \"%s\" passwd entry failed (%s).", username, strerror(ret));
+            ERR(NULL, "Retrieving username \"%s\" passwd entry failed (%s).", username,
+                    ret ? strerror(ret) : "user not found");
         } else {
-            ERR(NULL, "Retrieving UID \"%lu\" passwd entry failed (%s).", (unsigned long)uid, strerror(ret));
+            ERR(NULL, "Retrieving UID \"%lu\" passwd entry failed (%s).", (unsigned long)uid,
+                    ret ? strerror(ret) : "uid not found");
         }
     }
     return pwd;
 }
+
+struct group *
+nc_getgr(gid_t gid, const char *grpname, struct group *grp_buf, char **buf, size_t *buf_size)
+{
+    struct group *grp = NULL;
+    long sys_size;
+    int ret;
+
+    do {
+        if (!*buf_size) {
+            /* learn suitable buffer size */
+            sys_size = sysconf(_SC_GETGR_R_SIZE_MAX);
+            *buf_size = (sys_size == -1) ? 2048 : sys_size;
+        } else {
+            /* enlarge buffer */
+            *buf_size += 2048;
+        }
+
+        /* allocate some buffer */
+        *buf = nc_realloc(*buf, *buf_size);
+        NC_CHECK_ERRMEM_RET(!*buf, NULL);
+
+        if (grpname) {
+            ret = getgrnam_r(grpname, grp_buf, *buf, *buf_size, &grp);
+        } else {
+            ret = getgrgid_r(gid, grp_buf, *buf, *buf_size, &grp);
+        }
+    } while (ret && (ret == ERANGE));
+
+    if (ret || !grp) {
+        if (grpname) {
+            ERR(NULL, "Retrieving group \"%s\" group entry failed (%s).", grpname,
+                    ret ? strerror(ret) : "group not found");
+        } else {
+            ERR(NULL, "Retrieving GID \"%lu\" group entry failed (%s).", (unsigned long)gid,
+                    ret ? strerror(ret) : "gid not found");
+        }
+    }
+    return grp;
+}
