@@ -4,7 +4,7 @@
  * @brief libnetconf2 - server NETCONF messages functions
  *
  * @copyright
- * Copyright (c) 2015 CESNET, z.s.p.o.
+ * Copyright (c) 2015 - 2025 CESNET, z.s.p.o.
  *
  * This source code is licensed under BSD 3-Clause License (the "License").
  * You may not use this file except in compliance with the License.
@@ -261,14 +261,21 @@ nc_err(const struct ly_ctx *ctx, NC_ERR tag, ...)
 {
     va_list ap;
     struct lyd_node *err = NULL;
+    const struct lys_module *nc_mod;
     NC_ERR_TYPE type;
     const char *arg1, *arg2;
     uint32_t sid;
 
-    NC_CHECK_ARG_RET(NULL, tag, NULL);
+    NC_CHECK_ARG_RET(NULL, tag, ctx, NULL);
+
+    nc_mod = ly_ctx_get_module_implemented(ctx, "ietf-netconf");
+    if (!nc_mod) {
+        ERR(NULL, "Module \"ietf-netconf\" missing in the context.");
+        return NULL;
+    }
 
     /* rpc-error */
-    if (lyd_new_opaq2(NULL, ctx, "rpc-error", NULL, NULL, NC_NS_BASE, &err)) {
+    if (lyd_new_inner(NULL, nc_mod, "rpc-error", 0, &err)) {
         return NULL;
     }
 
@@ -337,17 +344,17 @@ nc_err(const struct ly_ctx *ctx, NC_ERR tag, ...)
         ERRARG(NULL, "tag");
         goto fail;
     }
-    if (lyd_new_opaq2(err, NULL, "error-type", nc_err_type2str(type), NULL, NC_NS_BASE, NULL)) {
+    if (lyd_new_term(err, NULL, "error-type", nc_err_type2str(type), 0, NULL)) {
         goto fail;
     }
 
     /* error-tag */
-    if (lyd_new_opaq2(err, NULL, "error-tag", nc_err_tag2str(tag), NULL, NC_NS_BASE, NULL)) {
+    if (lyd_new_term(err, NULL, "error-tag", nc_err_tag2str(tag), 0, NULL)) {
         goto fail;
     }
 
     /* error-severity */
-    if (lyd_new_opaq2(err, NULL, "error-severity", "error", NULL, NC_NS_BASE, NULL)) {
+    if (lyd_new_term(err, NULL, "error-severity", "error", 0, NULL)) {
         goto fail;
     }
 
@@ -474,13 +481,17 @@ fail:
 API NC_ERR_TYPE
 nc_err_get_type(const struct lyd_node *err)
 {
-    struct lyd_node *match;
+    const struct lysc_node *schema;
+    struct lyd_node *match = NULL;
 
     NC_CHECK_ARG_RET(NULL, err, 0);
 
-    lyd_find_sibling_opaq_next(lyd_child(err), "error-type", &match);
+    schema = lys_find_path(NULL, err->schema, "error-type", 0);
+    if (schema) {
+        lyd_find_sibling_val(lyd_child(err), schema, NULL, 0, &match);
+    }
     if (match) {
-        return nc_err_str2type(((struct lyd_node_opaq *)match)->value);
+        return nc_err_str2type(lyd_get_value(match));
     }
 
     return 0;
@@ -489,13 +500,17 @@ nc_err_get_type(const struct lyd_node *err)
 API NC_ERR
 nc_err_get_tag(const struct lyd_node *err)
 {
-    struct lyd_node *match;
+    const struct lysc_node *schema;
+    struct lyd_node *match = NULL;
 
     NC_CHECK_ARG_RET(NULL, err, 0);
 
-    lyd_find_sibling_opaq_next(lyd_child(err), "error-tag", &match);
+    schema = lys_find_path(NULL, err->schema, "error-tag", 0);
+    if (schema) {
+        lyd_find_sibling_val(lyd_child(err), schema, NULL, 0, &match);
+    }
     if (match) {
-        return nc_err_str2tag(((struct lyd_node_opaq *)match)->value);
+        return nc_err_str2tag(lyd_get_value(match));
     }
 
     return 0;
@@ -504,17 +519,25 @@ nc_err_get_tag(const struct lyd_node *err)
 API int
 nc_err_set_app_tag(struct lyd_node *err, const char *error_app_tag)
 {
+    const struct lysc_node *schema;
     struct lyd_node *match;
 
     NC_CHECK_ARG_RET(NULL, err, error_app_tag, -1);
 
+    /* find the schema node */
+    schema = lys_find_path(NULL, err->schema, "error-app-tag", 0);
+    if (!schema) {
+        return -1;
+    }
+
     /* remove previous node */
-    lyd_find_sibling_opaq_next(lyd_child(err), "error-app-tag", &match);
+    lyd_find_sibling_val(lyd_child(err), schema, NULL, 0, &match);
     if (match) {
         lyd_free_tree(match);
     }
 
-    if (lyd_new_opaq2(err, NULL, "error-app-tag", error_app_tag, NULL, NC_NS_BASE, NULL)) {
+    /* create the node */
+    if (lyd_new_term(err, NULL, "error-app-tag", error_app_tag, 0, &match)) {
         return -1;
     }
 
@@ -524,13 +547,17 @@ nc_err_set_app_tag(struct lyd_node *err, const char *error_app_tag)
 API const char *
 nc_err_get_app_tag(const struct lyd_node *err)
 {
-    struct lyd_node *match;
+    const struct lysc_node *schema;
+    struct lyd_node *match = NULL;
 
     NC_CHECK_ARG_RET(NULL, err, NULL);
 
-    lyd_find_sibling_opaq_next(lyd_child(err), "error-app-tag", &match);
+    schema = lys_find_path(NULL, err->schema, "error-tag", 0);
+    if (schema) {
+        lyd_find_sibling_val(lyd_child(err), schema, NULL, 0, &match);
+    }
     if (match) {
-        return ((struct lyd_node_opaq *)match)->value;
+        return lyd_get_value(match);
     }
 
     return NULL;
@@ -539,17 +566,25 @@ nc_err_get_app_tag(const struct lyd_node *err)
 API int
 nc_err_set_path(struct lyd_node *err, const char *error_path)
 {
+    const struct lysc_node *schema;
     struct lyd_node *match;
 
     NC_CHECK_ARG_RET(NULL, err, error_path, -1);
 
+    /* find the schema node */
+    schema = lys_find_path(NULL, err->schema, "error-path", 0);
+    if (!schema) {
+        return -1;
+    }
+
     /* remove previous node */
-    lyd_find_sibling_opaq_next(lyd_child(err), "error-path", &match);
+    lyd_find_sibling_val(lyd_child(err), schema, NULL, 0, &match);
     if (match) {
         lyd_free_tree(match);
     }
 
-    if (lyd_new_opaq2(err, NULL, "error-path", error_path, NULL, NC_NS_BASE, NULL)) {
+    /* create the node */
+    if (lyd_new_term(err, NULL, "error-path", error_path, 0, &match)) {
         return -1;
     }
 
@@ -559,13 +594,17 @@ nc_err_set_path(struct lyd_node *err, const char *error_path)
 API const char *
 nc_err_get_path(const struct lyd_node *err)
 {
-    struct lyd_node *match;
+    const struct lysc_node *schema;
+    struct lyd_node *match = NULL;
 
     NC_CHECK_ARG_RET(NULL, err, NULL);
 
-    lyd_find_sibling_opaq_next(lyd_child(err), "error-path", &match);
+    schema = lys_find_path(NULL, err->schema, "error-path", 0);
+    if (schema) {
+        lyd_find_sibling_val(lyd_child(err), schema, NULL, 0, &match);
+    }
     if (match) {
-        return ((struct lyd_node_opaq *)match)->value;
+        return lyd_get_value(match);
     }
 
     return NULL;
@@ -574,21 +613,37 @@ nc_err_get_path(const struct lyd_node *err)
 API int
 nc_err_set_msg(struct lyd_node *err, const char *error_message, const char *lang)
 {
-    struct lyd_node *match;
+    struct lyd_node *match, *prev_anchor;
     struct lyd_attr *attr;
 
     NC_CHECK_ARG_RET(NULL, err, error_message, -1);
 
+    /* remove previous node */
     lyd_find_sibling_opaq_next(lyd_child(err), "error-message", &match);
     if (match) {
-        /* Change the value of error-message and keep order of elements to comply with appendix-B in RFC 6241. */
-        lydict_remove(LYD_CTX(err), ((struct lyd_node_opaq *)match)->value);
-        lydict_insert(LYD_CTX(err), error_message, 0, &(((struct lyd_node_opaq *)match)->value));
-        return 0;
+        lyd_free_tree(match);
     }
+
+    /* find the previous node anchor (last non-opaque node) */
+    prev_anchor = lyd_child(err);
+    while (prev_anchor && prev_anchor->next && prev_anchor->next->schema) {
+        prev_anchor = prev_anchor->next;
+    }
+    if (!prev_anchor->schema) {
+        prev_anchor = NULL;
+    }
+
+    /* create the node at the right place */
     if (lyd_new_opaq2(err, NULL, "error-message", error_message, NULL, NC_NS_BASE, &match)) {
         return -1;
     }
+    if (prev_anchor) {
+        lyd_insert_after(prev_anchor, match);
+    } else if (match->prev != match) {
+        /* some opaque nodes existed, this must be the first */
+        lyd_insert_before(lyd_child(err), match);
+    }
+
     if (lang && lyd_new_attr(match, NULL, "xml:lang", lang, &attr)) {
         lyd_free_tree(match);
         return -1;
@@ -733,6 +788,16 @@ nc_server_rpc_free(struct nc_server_rpc *rpc)
     free(rpc);
 }
 
+API NC_RPL
+nc_server_reply_type(struct nc_server_reply *reply)
+{
+    if (!reply) {
+        return 0;
+    }
+
+    return reply->type;
+}
+
 API void
 nc_server_reply_free(struct nc_server_reply *reply)
 {
@@ -747,7 +812,7 @@ nc_server_reply_free(struct nc_server_reply *reply)
     case NC_RPL_DATA:
         data_rpl = (struct nc_server_reply_data *)reply;
         if (data_rpl->free) {
-            lyd_free_siblings(data_rpl->data);
+            lyd_free_all(data_rpl->data);
         }
         break;
     case NC_RPL_OK:
@@ -766,7 +831,7 @@ nc_server_reply_free(struct nc_server_reply *reply)
 API struct nc_server_notif *
 nc_server_notif_new(struct lyd_node *event, char *eventtime, NC_PARAMTYPE paramtype)
 {
-    struct nc_server_notif *ntf;
+    struct nc_server_notif *ntf = NULL;
     struct lyd_node *elem;
     int found;
 
@@ -791,9 +856,10 @@ nc_server_notif_new(struct lyd_node *event, char *eventtime, NC_PARAMTYPE paramt
 
     if (paramtype == NC_PARAMTYPE_DUP_AND_FREE) {
         ntf->eventtime = strdup(eventtime);
+        NC_CHECK_ERRMEM_GOTO(!ntf->eventtime, , error);
+
         if (lyd_dup_single(event, NULL, LYD_DUP_RECURSIVE, &ntf->ntf)) {
-            free(ntf);
-            return NULL;
+            goto error;
         }
     } else {
         ntf->eventtime = eventtime;
@@ -802,6 +868,11 @@ nc_server_notif_new(struct lyd_node *event, char *eventtime, NC_PARAMTYPE paramt
     ntf->free = (paramtype == NC_PARAMTYPE_CONST ? 0 : 1);
 
     return ntf;
+
+error:
+    free(ntf->eventtime);
+    free(ntf);
+    return NULL;
 }
 
 API void
