@@ -74,6 +74,9 @@ int nc_server_config_load_modules(struct ly_ctx **ctx);
  *
  * Context must already have implemented the required modules, see ::nc_server_config_load_modules().
  *
+ * @note All the implicit nodes, such as non-presence containers and
+ * leafs with default values, must be present in the diff.
+ *
  * @param[in] diff YANG diff belonging to either ietf-netconf-server, ietf-keystore or ietf-truststore modules.
  * The top level node HAS to have an operation (create, replace, delete or none).
  * @return 0 on success, 1 on error.
@@ -83,10 +86,13 @@ int nc_server_config_setup_diff(const struct lyd_node *diff);
 /**
  * @brief Configure server based on the given data.
  *
- * Behaves as if all the nodes in data had the replace operation. That means that the current configuration will be deleted
- * and just the given data will be applied.
+ * The current configuration will be completely replaced by the given data.
+ *
  * Context must already have implemented the required modules, see ::nc_server_config_load_modules().
  *
+ * @note All the implicit nodes, such as non-presence containers and
+ * leafs with default values, must be present in the data.
+ *
  * @param[in] data YANG data belonging to either ietf-netconf-server, ietf-keystore or ietf-truststore modules.
  * This data __must be valid__. No node can have an operation attribute.
  * @return 0 on success, 1 on error.
@@ -136,6 +142,72 @@ int nc_server_config_add_address_port(const struct ly_ctx *ctx, const char *endp
  */
 int nc_server_config_del_endpt(const char *endpt_name, struct lyd_node **config);
 
+/**
+ * @brief Creates new YANG data nodes for a UNIX socket endpoint.
+ *
+ * To learn how authentication works, look at ::nc_server_config_add_unix_user_mapping().
+ *
+ * @param[in] ctx libyang context.
+ * @param[in] endpt_name Arbitrary identifier of the endpoint.
+ * If an endpoint with this identifier already exists, its contents might be changed.
+ * @param[in] path Path to the UNIX socket. The parent directory must be writable by the server process.
+ * @param[in] mode Optional UNIX socket file mode in octal format (e.g., "0777").
+ * If NULL, the default value ("0660") will be used.
+ * @param[in] owner Optional owner of the UNIX socket file.
+ * If NULL, the owner will be set to the user running the server process.
+ * @param[in] group Optional group of the UNIX socket file.
+ * If NULL, the group will be set to the group of the user running the server process.
+ * @param[in,out] config Configuration YANG data tree. If *config is NULL, it will be created.
+ * Otherwise the new YANG data will be added to the previous data and may override it.
+ * @return 0 on success, non-zero otherwise.
+ *
+ */
+int nc_server_config_add_unix_socket(const struct ly_ctx *ctx, const char *endpt_name, const char *path,
+        const char *mode, const char *owner, const char *group, struct lyd_node **config);
+
+/**
+ * @brief Creates new YANG data nodes for a UNIX username mapping.
+ *
+ * Specifies who can connect to a UNIX socket endpoint. There are 3 following scenarios:
+ *
+ * 1) If no mappings are configured for the client @p system_user, then the client can
+ * only connect if its username matches @p netconf_user.
+ *
+ * 2) If a mapping for the client @p system_user is configured, but no @p netconf_user s
+ * are specified, then the client cannot connect with any username.
+ *
+ * 3) If a mapping for the client @p system_user is configured, and at least one @p netconf_user
+ * is specified, then the client can connect only with one of the specified usernames.
+ * If a wildcard value "*" is configured for @p netconf_user, then the client can connect
+ * with any NETCONF username.
+ *
+ * @param[in] ctx libyang context.
+ * @param[in] endpt_name Arbitrary identifier of the endpoint.
+ * If an endpoint with this identifier already exists, its contents might be changed.
+ * @param[in] system_user System user name, who the mapping will be created for.
+ * @param[in] netconf_user Username that the @p system_user can use to connect to the endpoint.
+ * Special value "*" can be used to allow any username.
+ * If NULL, empty mapping will be created, which means that the @p system_user
+ * can not connect to the endpoint with any username.
+ * @param[in,out] config Configuration YANG data tree. If *config is NULL, it will be created.
+ * Otherwise the new YANG data will be added to the previous data and may override it.
+ * @return 0 on success, non-zero otherwise.
+ */
+int nc_server_config_add_unix_user_mapping(const struct ly_ctx *ctx, const char *endpt_name,
+        const char *system_user, const char *netconf_user, struct lyd_node **config);
+
+/**
+ * @brief Deletes a UNIX username mapping from the YANG data.
+ *
+ * @param[in] endpt_name Identifier of an existing endpoint.
+ * @param[in] system_user Optional identifier of a system user whose mapping will be deleted.
+ * If NULL, all of the mappings for the endpoint will be deleted.
+ * @param[in,out] config Configuration YANG data tree.
+ * @return 0 on success, non-zero otherwise.
+ */
+int nc_server_config_del_unix_user_mapping(const char *endpt_name, const char *system_user,
+        struct lyd_node **config);
+
 #ifdef NC_ENABLED_SSH_TLS
 
 /**
@@ -254,40 +326,39 @@ int nc_server_config_del_truststore_cert(const char *cert_bag_name,
         const char *cert_name, struct lyd_node **config);
 
 /**
- * @brief Gets the hostkey algorithms supported by the server from the 'iana-ssh-public-key-algs' YANG module.
+ * @brief Gets all the SSH transport algorithms supported by the server.
+ *
+ * @note Actual usable algorithms depend on the underlying libssh library version,
+ * the returned list is based on libssh v0.11.0.
  *
  * @param[in] ctx libyang context.
- * @param[out] hostkey_algs Container with leaf-lists containing the supported algorithms.
+ * @param[out] supported_algs Supported SSH transport algorithms.
  * @return 0 on success, non-zero otherwise.
  */
-int nc_server_config_oper_get_hostkey_algs(const struct ly_ctx *ctx, struct lyd_node **hostkey_algs);
+int nc_server_config_oper_get_supported_ssh_algs(const struct ly_ctx *ctx, struct lyd_node **supported_algs);
 
 /**
- * @brief Gets the key exchange algorithms supported by the server from the 'iana-ssh-key-exchange-algs' YANG module.
+ * @brief Gets all the TLS transport cipher suites supported by the server.
+ *
+ * @note Actual usable cipher suites depend on the underlying TLS library and its version.
  *
  * @param[in] ctx libyang context.
- * @param[out] kex_algs Container with leaf-lists containing the supported algorithms.
+ * @param[out] supported_algs Supported cipher suites.
  * @return 0 on success, non-zero otherwise.
  */
-int nc_server_config_oper_get_kex_algs(const struct ly_ctx *ctx, struct lyd_node **kex_algs);
+int nc_server_config_oper_get_supported_tls_algs(const struct ly_ctx *ctx, struct lyd_node **supported_algs);
 
 /**
- * @brief Gets the encryption algorithms supported by the server from the 'iana-ssh-encryption-algs' YANG module.
+ * @brief Gets the last modified time of an SSH user's password.
  *
- * @param[in] ctx libyang context.
- * @param[out] encryption_algs Container with leaf-lists containing the supported algorithms.
+ * @param[in] ch_client Name of the call-home client the user is configured on. NULL if the user is on a listening endpoint.
+ * @param[in] endpoint Name of the endpoint the user is configured on. Can be either a listening or call-home endpoint.
+ * @param[in] username Name of the SSH user.
+ * @param[out] last_modified Time of the last password modification.
  * @return 0 on success, non-zero otherwise.
  */
-int nc_server_config_oper_get_encryption_algs(const struct ly_ctx *ctx, struct lyd_node **encryption_algs);
-
-/**
- * @brief Gets the MAC algorithms supported by the server from the 'iana-ssh-mac-algs' YANG module.
- *
- * @param[in] ctx libyang context.
- * @param[out] mac_algs Container with leaf-lists containing the supported algorithms.
- * @return 0 on success, non-zero otherwise.
- */
-int nc_server_config_oper_get_mac_algs(const struct ly_ctx *ctx, struct lyd_node **mac_algs);
+int nc_server_config_oper_get_user_password_last_modified(const char *ch_client,
+        const char *endpoint, const char *username, time_t *last_modified);
 
 /**
  * @} Server Configuration Functions
