@@ -1,10 +1,11 @@
 /**
  * @file server_config_util.c
  * @author Roman Janota <janota@cesnet.cz>
+ * @author Michal Vasko <mvasko@cesnet.cz>
  * @brief libnetconf2 server configuration utilities
  *
  * @copyright
- * Copyright (c) 2023 CESNET, z.s.p.o.
+ * Copyright (c) 2023 - 2025 CESNET, z.s.p.o.
  *
  * This source code is licensed under BSD 3-Clause License (the "License").
  * You may not use this file except in compliance with the License.
@@ -40,6 +41,7 @@ int
 nc_server_config_create(const struct ly_ctx *ctx, struct lyd_node **tree, const char *value, const char *path_fmt, ...)
 {
     int ret = 0;
+    struct lyd_node *iter;
     va_list ap;
     char *path = NULL;
 
@@ -68,10 +70,12 @@ nc_server_config_create(const struct ly_ctx *ctx, struct lyd_node **tree, const
         goto cleanup;
     }
 
-    /* add all default nodes */
-    ret = lyd_new_implicit_tree(*tree, LYD_IMPLICIT_NO_STATE, NULL);
-    if (ret) {
-        goto cleanup;
+    /* add all implicit nodes of ietf-netconf-server, ietf-keystore and ietf-truststore */
+    LY_LIST_FOR(lyd_first_sibling(*tree), iter) {
+        ret = lyd_new_implicit_tree(iter, LYD_IMPLICIT_NO_STATE, NULL);
+        if (ret) {
+            goto cleanup;
+        }
     }
 
 cleanup:
@@ -85,6 +89,7 @@ nc_server_config_append(const struct ly_ctx *ctx, const char *parent_path, const
         const char *value, struct lyd_node **tree)
 {
     int ret = 0;
+    struct lyd_node *iter;
     char *path = NULL;
 
     NC_CHECK_ARG_RET(NULL, ctx, parent_path, child_name, tree, 1);
@@ -110,10 +115,12 @@ nc_server_config_append(const struct ly_ctx *ctx, const char *parent_path, const
         goto cleanup;
     }
 
-    /* add all default nodes */
-    ret = lyd_new_implicit_tree(*tree, LYD_IMPLICIT_NO_STATE, NULL);
-    if (ret) {
-        goto cleanup;
+    /* add all implicit nodes of ietf-netconf-server, ietf-keystore and ietf-truststore */
+    LY_LIST_FOR(lyd_first_sibling(*tree), iter) {
+        ret = lyd_new_implicit_tree(iter, LYD_IMPLICIT_NO_STATE, NULL);
+        if (ret) {
+            goto cleanup;
+        }
     }
 
 cleanup:
@@ -127,7 +134,7 @@ nc_server_config_delete(struct lyd_node **tree, const char *path_fmt, ...)
     int ret = 0;
     va_list ap;
     char *path = NULL;
-    struct lyd_node *sub = NULL;
+    struct lyd_node *sub = NULL, *iter;
 
     NC_CHECK_ARG_RET(NULL, tree, path_fmt, 1);
 
@@ -145,16 +152,18 @@ nc_server_config_delete(struct lyd_node **tree, const char *path_fmt, ...)
 
     lyd_free_tree(sub);
 
-    /* set the node to top level container */
+    /* set the node to the top level node */
     ret = lyd_find_path(*tree, "/ietf-netconf-server:netconf-server", 0, tree);
     if (ret) {
         goto cleanup;
     }
 
-    /* add all default nodes */
-    ret = lyd_new_implicit_tree(*tree, LYD_IMPLICIT_NO_STATE, NULL);
-    if (ret) {
-        goto cleanup;
+    /* add all implicit nodes of ietf-netconf-server, ietf-keystore and ietf-truststore */
+    LY_LIST_FOR(lyd_first_sibling(*tree), iter) {
+        ret = lyd_new_implicit_tree(iter, LYD_IMPLICIT_NO_STATE, NULL);
+        if (ret) {
+            goto cleanup;
+        }
     }
 
 cleanup:
@@ -169,7 +178,7 @@ nc_server_config_check_delete(struct lyd_node **tree, const char *path_fmt, ...)
     int ret = 0;
     va_list ap;
     char *path = NULL;
-    struct lyd_node *sub = NULL;
+    struct lyd_node *sub = NULL, *iter;
 
     NC_CHECK_ARG_RET(NULL, tree, path_fmt, 1);
 
@@ -191,18 +200,139 @@ nc_server_config_check_delete(struct lyd_node **tree, const char *path_fmt, ...)
 
     lyd_free_tree(sub);
 
-    /* set the node to top level container */
+    /* set the node to the top level node */
     ret = lyd_find_path(*tree, "/ietf-netconf-server:netconf-server", 0, tree);
     if (ret) {
         goto cleanup;
     }
 
+    /* add all implicit nodes of ietf-netconf-server, ietf-keystore and ietf-truststore */
+    LY_LIST_FOR(lyd_first_sibling(*tree), iter) {
+        ret = lyd_new_implicit_tree(iter, LYD_IMPLICIT_NO_STATE, NULL);
+        if (ret) {
+            goto cleanup;
+        }
+    }
+
 cleanup:
     free(path);
     va_end(ap);
     return ret;
 }
 
+API int
+nc_server_config_del_endpt(const char *endpt_name, struct lyd_node **config)
+{
+    NC_CHECK_ARG_RET(NULL, config, 1);
+
+    if (endpt_name) {
+        return nc_server_config_delete(config, "/ietf-netconf-server:netconf-server/listen/endpoints/endpoint[name='%s']", endpt_name);
+    } else {
+        return nc_server_config_delete(config, "/ietf-netconf-server:netconf-server/listen/endpoints/endpoint");
+    }
+}
+
+API int
+nc_server_config_add_unix_socket(const struct ly_ctx *ctx, const char *endpt_name, const char *path,
+        const char *mode, const char *owner, const char *group, struct lyd_node **config)
+{
+    int rc = 0;
+    char *path_fmt = NULL;
+    struct lys_module *mod;
+    const char *path_type_str;
+
+    NC_CHECK_ARG_RET(NULL, ctx, config, 1);
+
+    if (!path) {
+        /* creating a hidden UNIX socket path set by other means */
+        path_type_str = "hidden-path";
+    } else {
+        /* creating a standard UNIX socket path */
+        path_type_str = "socket-path";
+
+        /* check if the 'unix-socket-path' feature is enabled in the libnetconf2-netconf-server module */
+        mod = ly_ctx_get_module_implemented(ctx, "libnetconf2-netconf-server");
+        NC_CHECK_RET(!mod, 1);
+        if (lys_feature_value(mod, "unix-socket-path")) {
+            ERR(NULL, "Unable to set UNIX socket path ('unix-socket-path' feature not enabled in "
+                    "'libnetconf2-netconf-server' module).");
+            return 1;
+        }
+    }
+
+    /* create the path to UNIX socket or just the empty leaf for hidden path */
+    NC_CHECK_ERR_RET(asprintf(&path_fmt, "/ietf-netconf-server:netconf-server/listen/endpoints/endpoint[name='%s']/"
+            "libnetconf2-netconf-server:unix/%s", endpt_name, path_type_str) == -1, ERRMEM, 1);
+    NC_CHECK_GOTO(rc = nc_server_config_create(ctx, config, path, path_fmt), cleanup);
+
+    if (!mode && !owner && !group) {
+        /* nothing more to do */
+        goto cleanup;
+    }
+
+    /* create the path to socket's mode/owner/group */
+    free(path_fmt);
+    NC_CHECK_ERR_RET(asprintf(&path_fmt, "/ietf-netconf-server:netconf-server/listen/endpoints/endpoint[name='%s']/"
+            "libnetconf2-netconf-server:unix/socket-permissions", endpt_name) == -1, ERRMEM, 1);
+
+    if (mode) {
+        /* set the mode */
+        NC_CHECK_GOTO(rc = nc_server_config_append(ctx, path_fmt, "mode", mode, config), cleanup);
+    }
+    if (owner) {
+        /* set the owner */
+        NC_CHECK_GOTO(rc = nc_server_config_append(ctx, path_fmt, "owner", owner, config), cleanup);
+    }
+    if (group) {
+        /* set the group */
+        NC_CHECK_GOTO(rc = nc_server_config_append(ctx, path_fmt, "group", group, config), cleanup);
+    }
+
+cleanup:
+    free(path_fmt);
+    return rc;
+}
+
+API int
+nc_server_config_add_unix_user_mapping(const struct ly_ctx *ctx, const char *endpt_name,
+        const char *system_user, const char *netconf_user, struct lyd_node **config)
+{
+    int rc = 0;
+    char *path_fmt = NULL;
+
+    NC_CHECK_ARG_RET(NULL, endpt_name, system_user, config, 1);
+
+    NC_CHECK_ERR_RET(asprintf(&path_fmt, "/ietf-netconf-server:netconf-server/listen/endpoints/endpoint[name='%s']/"
+            "libnetconf2-netconf-server:unix/client-authentication/user-mapping[system-user='%s']/netconf-user",
+            endpt_name, system_user) == -1, ERRMEM, 1);
+
+    if (netconf_user) {
+        NC_CHECK_GOTO(rc = nc_server_config_create(ctx, config, netconf_user, path_fmt), cleanup);
+    } else {
+        NC_CHECK_GOTO(rc = nc_server_config_create(ctx, config, NULL, path_fmt), cleanup);
+    }
+
+cleanup:
+    free(path_fmt);
+    return rc;
+}
+
+API int
+nc_server_config_del_unix_user_mapping(const char *endpt_name, const char *system_user, struct lyd_node **config)
+{
+    NC_CHECK_ARG_RET(NULL, config, 1);
+
+    if (system_user) {
+        return nc_server_config_delete(config, "/ietf-netconf-server:netconf-server/listen/endpoints/endpoint[name='%s']/"
+                "libnetconf2-netconf-server:unix/client-authentication/user-mapping[system-user='%s']",
+                endpt_name, system_user);
+    } else {
+        /* delete all instances */
+        return nc_server_config_delete(config, "/ietf-netconf-server:netconf-server/listen/endpoints/endpoint[name='%s']/"
+                "libnetconf2-netconf-server:unix/client-authentication/user-mapping", endpt_name);
+    }
+}
+
 #ifdef NC_ENABLED_SSH_TLS
 
 const char *
@@ -893,32 +1023,27 @@ nc_server_config_add_address_port(const struct ly_ctx *ctx, const char *endpt_na
         const char *address, uint16_t port, struct lyd_node **config)
 {
     int ret = 0;
-    const char *address_fmt, *port_fmt;
+    const char *path_fmt, *transport_str;
     char port_buf[6] = {0};
 
     NC_CHECK_ARG_RET(NULL, ctx, endpt_name, address, config, 1);
 
+    /* prepare the path */
+    path_fmt = "/ietf-netconf-server:netconf-server/listen/endpoints/endpoint[name='%s']/%s/tcp-server-parameters/"
+            "local-bind[local-address='%s']/local-port";
+
     if (transport == NC_TI_SSH) {
-        /* SSH path */
-        address_fmt = "/ietf-netconf-server:netconf-server/listen/endpoints/endpoint[name='%s']/ssh/tcp-server-parameters/local-address";
-        port_fmt = "/ietf-netconf-server:netconf-server/listen/endpoints/endpoint[name='%s']/ssh/tcp-server-parameters/local-port";
+        transport_str = "ssh";
     } else if (transport == NC_TI_TLS) {
-        /* TLS path */
-        address_fmt = "/ietf-netconf-server:netconf-server/listen/endpoints/endpoint[name='%s']/tls/tcp-server-parameters/local-address";
-        port_fmt = "/ietf-netconf-server:netconf-server/listen/endpoints/endpoint[name='%s']/tls/tcp-server-parameters/local-port";
+        transport_str = "tls";
     } else {
         ERR(NULL, "Can not set address and port of a non SSH/TLS endpoint.");
         ret = 1;
         goto cleanup;
     }
 
-    ret = nc_server_config_create(ctx, config, address, address_fmt, endpt_name);
-    if (ret) {
-        goto cleanup;
-    }
-
-    sprintf(port_buf, "%d", port);
-    ret = nc_server_config_create(ctx, config, port_buf, port_fmt, endpt_name);
+    snprintf(port_buf, sizeof(port_buf), "%u", port);
+    ret = nc_server_config_create(ctx, config, port_buf, path_fmt, endpt_name, transport_str, address);
     if (ret) {
         goto cleanup;
     }
@@ -964,18 +1089,6 @@ cleanup:
     return ret;
 }
 
-API int
-nc_server_config_del_endpt(const char *endpt_name, struct lyd_node **config)
-{
-    NC_CHECK_ARG_RET(NULL, config, 1);
-
-    if (endpt_name) {
-        return nc_server_config_delete(config, "/ietf-netconf-server:netconf-server/listen/endpoints/endpoint[name='%s']", endpt_name);
-    } else {
-        return nc_server_config_delete(config, "/ietf-netconf-server:netconf-server/listen/endpoints/endpoint");
-    }
-}
-
 API int
 nc_server_config_del_ch_client(const char *ch_client_name, struct lyd_node **config)
 {
