@@ -567,7 +567,7 @@ nc_session_get_version(const struct nc_session *session)
 {
     NC_CHECK_ARG_RET(session, session, -1);
 
-    return session->version == NC_VERSION_10 ? 0 : 1;
+    return session->version == NC_PROT_VERSION_10 ? 0 : 1;
 }
 
 API NC_TRANSPORT_IMPL
@@ -720,7 +720,7 @@ read_msg:
             ly_in_free(msg, 1);
             goto read_msg;
         }
-        if (lyd_parse_op(session->ctx, close_rpc, msg, LYD_XML, LYD_TYPE_REPLY_NETCONF, &envp, NULL)) {
+        if (lyd_parse_op(session->ctx, close_rpc, msg, LYD_XML, LYD_TYPE_REPLY_NETCONF, LYD_PARSE_STRICT, &envp, NULL)) {
             WRN(session, "Failed to parse <close-session> reply.");
         } else if (!lyd_child(envp) || strcmp(LYD_NAME(lyd_child(envp)), "ok")) {
             WRN(session, "Reply to <close-session> was not <ok> as expected.");
@@ -816,12 +816,18 @@ nc_session_free_transport(struct nc_session *session, int *multisession)
                     free(siter);
                 } while (session->ti.libssh.next != session);
             }
-            /* remember sock so we can close it */
-            sock = ssh_get_fd(session->ti.libssh.session);
             if (connected) {
-                /* does not close sock */
+                /* remember sock so we can close it */
+                sock = ssh_get_fd(session->ti.libssh.session);
+
+                /* clears sock but does not close it if passed via options (libssh >= 0.10) */
                 ssh_disconnect(session->ti.libssh.session);
+#if (LIBSSH_VERSION_MAJOR == 0 && LIBSSH_VERSION_MINOR < 10)
+                sock = -1;
+#endif
             }
+
+            /* closes sock if set */
             ssh_free(session->ti.libssh.session);
         } else {
             /* remove the session from the list */
@@ -983,26 +989,6 @@ nc_session_free(struct nc_session *session, void (*data_free)(void *))
             }
             free(session->opts.client.cpblts);
         }
-
-        /* LY ext data */
-#ifdef NC_ENABLED_SSH_TLS
-        struct nc_session *siter;
-
-        if ((session->flags & NC_SESSION_SHAREDCTX) && (session->ti_type == NC_TI_SSH) && session->ti.libssh.next) {
-            for (siter = session->ti.libssh.next; siter != session; siter = siter->ti.libssh.next) {
-                if (siter->status != NC_STATUS_STARTING) {
-                    /* move LY ext data to this session */
-                    assert(!siter->opts.client.ext_data);
-                    siter->opts.client.ext_data = session->opts.client.ext_data;
-                    session->opts.client.ext_data = NULL;
-                    break;
-                }
-            }
-        } else
-#endif /* NC_ENABLED_SSH_TLS */
-        {
-            lyd_free_siblings(session->opts.client.ext_data);
-        }
     }
 
     if (session->data && data_free) {
@@ -1116,9 +1102,9 @@ nc_server_get_cpblts_version(const struct ly_ctx *ctx, LYS_VERSION version)
 {
     char **cpblts;
     const struct lys_module *mod;
-    struct lysp_feature *feat;
+    const char *feat;
     int size = 10, count, features_count = 0, dev_count = 0, str_len, len;
-    uint32_t i, u;
+    uint32_t u;
     LY_ARRAY_COUNT_TYPE v;
     char *yl_content_id;
     uint32_t wd_also_supported;
@@ -1259,40 +1245,35 @@ nc_server_get_cpblts_version(const struct ly_ctx *ctx, LYS_VERSION version)
             }
             free(yl_content_id);
             continue;
-        } else if ((version == LYS_VERSION_1_0) && (mod->parsed->version > version)) {
-            /* skip YANG 1.1 modules */
-            continue;
-        } else if ((version == LYS_VERSION_1_1) && (mod->parsed->version != version)) {
-            /* skip YANG 1.0 modules */
+        } else if (mod->version != version) {
+            /* skip YANG 1.0 or 1.1 modules */
             continue;
         }
 
         str_len = sprintf(str, "%s?module=%s%s%s", mod->ns, mod->name, mod->revision ? "&revision=" : "",
                 mod->revision ? mod->revision : "");
 
-        features_count = 0;
-        i = 0;
-        feat = NULL;
-        while ((feat = lysp_feature_next(feat, mod->parsed, &i))) {
-            if (!(feat->flags & LYS_FENABLED)) {
-                continue;
+        if (mod->compiled) {
+            features_count = 0;
+            LY_ARRAY_FOR(mod->compiled->features, v) {
+                feat = mod->compiled->features[v];
+                if (!features_count) {
+                    strcat(str, "&features=");
+                    str_len += 10;
+                }
+                len = strlen(feat);
+                if (str_len + 1 + len >= NC_CPBLT_BUF_LEN) {
+                    ERRINT;
+                    break;
+                }
+                if (features_count) {
+                    strcat(str, ",");
+                    ++str_len;
+                }
+                strcat(str, feat);
+                str_len += len;
+                features_count++;
             }
-            if (!features_count) {
-                strcat(str, "&features=");
-                str_len += 10;
-            }
-            len = strlen(feat->name);
-            if (str_len + 1 + len >= NC_CPBLT_BUF_LEN) {
-                ERRINT;
-                break;
-            }
-            if (features_count) {
-                strcat(str, ",");
-                ++str_len;
-            }
-            strcat(str, feat->name);
-            str_len += len;
-            features_count++;
         }
 
         if (mod->deviated_by) {
@@ -1425,7 +1406,7 @@ nc_send_hello_io(struct nc_session *session)
         if (session->flags & NC_SESSION_CALLHOME) {
             timeout_io = NC_SERVER_CH_HELLO_TIMEOUT * 1000;
         } else {
-            timeout_io = server_opts.idle_timeout ? server_opts.idle_timeout * 1000 : -1;
+            timeout_io = server_opts.config.idle_timeout ? server_opts.config.idle_timeout * 1000 : -1;
         }
         sid = &session->id;
     }
@@ -1547,7 +1528,7 @@ nc_server_recv_hello_io(struct nc_session *session)
     if (session->flags & NC_SESSION_CALLHOME) {
         timeout_io = NC_SERVER_CH_HELLO_TIMEOUT * 1000;
     } else {
-        timeout_io = server_opts.idle_timeout ? server_opts.idle_timeout * 1000 : -1;
+        timeout_io = server_opts.config.idle_timeout ? server_opts.config.idle_timeout * 1000 : -1;
     }
 
     r = nc_read_msg_poll_io(session, timeout_io, &msg);
