@@ -4,7 +4,7 @@
  * @brief libnetconf2 session client manipulation
  *
  * @copyright
- * Copyright (c) 2015 - 2021 CESNET, z.s.p.o.
+ * Copyright (c) 2015 - 2025 CESNET, z.s.p.o.
  *
  * This source code is licensed under BSD 3-Clause License (the "License").
  * You may not use this file except in compliance with the License.
@@ -89,6 +89,14 @@ ly_module_imp_clb nc_client_get_schema_callback(void **user_data);
  */
 void nc_client_set_new_session_context_autofill(int enabled);
 
+/**
+ * @brief Set the NETCONF user of a UNIX socket connection. By default it is the system user of the client process.
+ *
+ * @param[in] username Username to authenticate as when creating a new UNIX socket NETCONF session.
+ * @return 0 on success, -1 on error.
+ */
+int nc_client_unix_set_username(const char *username);
+
 /**
  * @brief Set client session context to support schema-mount, if possible.
  *
@@ -449,21 +457,10 @@ int nc_client_tls_set_trusted_ca_paths(const char *ca_file, const char *ca_dir);
  */
 void nc_client_tls_get_trusted_ca_paths(const char **ca_file, const char **ca_dir);
 
-/**
- * @brief Deprecated.
- */
-int nc_client_tls_set_crl_paths(const char *crl_file, const char *crl_dir);
-
-/**
- * @brief Deprecated.
- */
-void nc_client_tls_get_crl_paths(const char **crl_file, const char **crl_dir);
-
 /**
  * @brief Connect to the NETCONF server using TLS transport (via libssl)
  *
- * TLS session is created with the certificates set using nc_client_tls_* functions, which must be called beforehand!
- * If the caller needs to use specific TLS session properties, they are supposed to use ::nc_connect_libssl().
+ * TLS session is created with the certificates set in the configuration.
  *
  * @param[in] host Hostname or address (both Ipv4 and IPv6 are accepted) of the target server.
  * 'localhost' is used by default if NULL is specified. It is verified by TLS when connecting to it.
@@ -475,11 +472,6 @@ void nc_client_tls_get_crl_paths(const char **crl_file, const char **crl_dir);
  */
 struct nc_session *nc_connect_tls(const char *host, uint16_t port, struct ly_ctx *ctx);
 
-/**
- * @brief Deprecated. Should not be needed.
- */
-struct nc_session *nc_connect_libssl(void *tls, struct ly_ctx *ctx);
-
 /** @} Client TLS */
 
 #endif /* NC_ENABLED_SSH_TLS */
