@@ -186,34 +186,6 @@ nc_client_tls_ch_get_trusted_ca_paths(const char **ca_file, const char **ca_dir)
     _nc_client_tls_get_trusted_ca_paths(ca_file, ca_dir, &tls_ch_opts);
 }
 
-API int
-nc_client_tls_set_crl_paths(const char *UNUSED(crl_file), const char *UNUSED(crl_dir))
-{
-    ERR(NULL, "nc_client_tls_set_crl_paths() is deprecated, do not use it.");
-    return -1;
-}
-
-API int
-nc_client_tls_ch_set_crl_paths(const char *UNUSED(crl_file), const char *UNUSED(crl_dir))
-{
-    ERR(NULL, "nc_client_tls_ch_set_crl_paths() is deprecated, do not use it.");
-    return -1;
-}
-
-API void
-nc_client_tls_get_crl_paths(const char **UNUSED(crl_file), const char **UNUSED(crl_dir))
-{
-    ERR(NULL, "nc_client_tls_get_crl_paths() is deprecated, do not use it.");
-    return;
-}
-
-API void
-nc_client_tls_ch_get_crl_paths(const char **UNUSED(crl_file), const char **UNUSED(crl_dir))
-{
-    ERR(NULL, "nc_client_tls_ch_get_crl_paths() is deprecated, do not use it.");
-    return;
-}
-
 API int
 nc_client_tls_ch_add_bind_listen(const char *address, uint16_t port)
 {
@@ -298,7 +270,7 @@ nc_client_tls_session_new(int sock, const char *host, int timeout, struct nc_cli
     }
 
     /* init TLS context and store data which may be needed later in it */
-    if (nc_tls_init_ctx_wrap(cli_cert, cli_pkey, cert_store, crl_store, tls_ctx)) {
+    if (nc_tls_init_ctx_wrap(cli_cert, cli_pkey, cert_store, crl_store, NULL, tls_ctx)) {
         goto fail;
     }
 
@@ -442,13 +414,6 @@ fail:
     return NULL;
 }
 
-API struct nc_session *
-nc_connect_libssl(void *UNUSED(tls), struct ly_ctx *UNUSED(ctx))
-{
-    ERR(NULL, "nc_connect_libssl() is deprecated, do not use it.");
-    return NULL;
-}
-
 struct nc_session *
 nc_accept_callhome_tls_sock(int sock, const char *host, uint16_t port, struct ly_ctx *ctx, int timeout, const char *peername)
 {
