@@ -384,33 +384,10 @@ void nc_ps_clear(struct nc_pollsession *ps, int all, void (*data_free)(void *));
 
 /**
  * @brief Get the number of currently configured listening endpoints.
- * Note that an ednpoint without address and/or port will be included
- * even though it is not, in fact, listening.
  *
- * @return Number of added listening endpoints.
+ * @return Number of listening endpoints.
  */
-int nc_server_endpt_count(void);
-
-/**
- * @brief Create a new UNIX socket endpoint and start listening.
- *
- * @param[in] endpt_name Arbitrary unique identifier of the endpoint.
- * @param[in] unix_socket_path Path to the listening socket.
- * @param[in] mode New mode, -1 to use default.
- * @param[in] uid New uid, -1 to use default.
- * @param[in] gid New gid, -1 to use default.
- *
- * @return 0 on success, 1 on error.
- */
-int nc_server_add_endpt_unix_socket_listen(const char *endpt_name, const char *unix_socket_path, mode_t mode, uid_t uid, gid_t gid);
-
-/**
- * @brief Deletes a UNIX socket endpoint.
- *
- * @param[in] endpt_name Identifier of the endpoint.
- * Has no effect if the endpoint doesn't exist or if its transport is not UNIX socket.
- */
-void nc_server_del_endpt_unix_socket(const char *endpt_name);
+uint32_t nc_server_endpt_count(void);
 
 /** @} */
 
@@ -470,6 +447,47 @@ NC_MSG_TYPE nc_session_accept_ssh_channel(struct nc_session *orig_session, struc
  */
 NC_MSG_TYPE nc_ps_accept_ssh_channel(struct nc_pollsession *ps, struct nc_session **session);
 
+/**
+ * @brief Set the UNIX socket path for a given endpoint name.
+ *
+ * The @p endpoint_name endpoint must be configured to use hidden UNIX socket path
+ * for this setting to take effect, see ::nc_server_config_add_unix_socket().
+ *
+ * @param[in] endpoint_name Name of the endpoint.
+ * @param[in] socket_path UNIX socket path to set.
+ * @return 0 on success, 1 on error.
+ */
+int nc_server_set_unix_socket_path(const char *endpoint_name, const char *socket_path);
+
+/**
+ * @brief Get the UNIX socket path for a given endpoint name.
+ *
+ * @param[in] endpoint_name Name of the endpoint.
+ * @param[out] socket_path Found UNIX socket path.
+ * @return 0 on success, 1 on error.
+ */
+int nc_server_get_unix_socket_path(const char *endpoint_name, char **socket_path);
+
+/**
+ * @brief Set the base directory for UNIX socket paths.
+ *
+ * All YANG defined UNIX socket paths will be relative to this directory.
+ * It must exist and be writable by the server process.
+ * This does not apply to "hidden" UNIX socket paths set via ::nc_server_set_unix_socket_path().
+ *
+ * @param[in] dir Base directory for UNIX socket paths.
+ * @return 0 on success, 1 on error.
+ */
+int nc_server_set_unix_socket_dir(const char *dir);
+
+/**
+ * @brief Get the base directory for UNIX socket paths.
+ *
+ * @param[out] dir Base directory for UNIX socket paths.
+ * @return 0 on success, 1 on error.
+ */
+int nc_server_get_unix_socket_dir(char **dir);
+
 /** @} Server Session */
 
 /**
